@@ -48,7 +48,7 @@ EXTERNL int ncbytesremove(NCbytes*,unsigned long);
 EXTERNL int ncbytescat(NCbytes*,const char*);
 
 /* Set the contents of the buffer; mark the buffer as non-extendible */
-EXTERNL int ncbytessetcontents(NCbytes*, char*, unsigned long);
+EXTERNL int ncbytessetcontents(NCbytes*, void*, unsigned long);
 
 /* Following are always "in-lined"*/
 #define ncbyteslength(bb) ((bb)!=NULL?(bb)->length:0)
@@ -57,6 +57,7 @@ EXTERNL int ncbytessetcontents(NCbytes*, char*, unsigned long);
 #define ncbytesextend(bb,len) ncbytessetalloc((bb),(len)+(bb->alloc))
 #define ncbytesclear(bb) ((bb)!=NULL?(bb)->length=0:0)
 #define ncbytesavail(bb,n) ((bb)!=NULL?((bb)->alloc - (bb)->length) >= (n):0)
+#define ncbytesextendible(bb) ((bb)!=NULL?((bb)->nonextendible?0:1):0)
 
 #if defined(_CPLUSPLUS_) || defined(__CPLUSPLUS__) || defined(__CPLUSPLUS)
 }
