@@ -35,7 +35,7 @@
 
 /* Given a filename, check its magic number */
 /* Change magic number size from 4 to 8 to be more precise for HDF5 */
-#define MAGIC_NUMBER_LEN ((size_t)8)
+#define MAGIC_NUMBER_LEN ((unsigned long long)8)
 #define MAGIC_HDF5_FILE 1
 #define MAGIC_HDF4_FILE 2
 #define MAGIC_CDF1_FILE 1 /* std classic format */
@@ -104,12 +104,12 @@ extern const NC_Dispatch* NC3_dispatch_table;
 extern int NC3_initialize(void);
 extern int NC3_finalize(void);
 
-#ifdef ENABLE_DAP
+#ifdef NETCDF_ENABLE_DAP
 extern const NC_Dispatch* NCD2_dispatch_table;
 extern int NCD2_initialize(void);
 extern int NCD2_finalize(void);
 #endif
-#ifdef ENABLE_DAP4
+#ifdef NETCDF_ENABLE_DAP4
 extern const NC_Dispatch* NCD4_dispatch_table;
 extern int NCD4_initialize(void);
 extern int NCD4_finalize(void);
@@ -138,7 +138,7 @@ extern int HDF4_initialize(void);
 extern int HDF4_finalize(void);
 #endif
 
-#ifdef ENABLE_NCZARR
+#ifdef NETCDF_ENABLE_NCZARR
 extern const NC_Dispatch* NCZ_dispatch_table;
 extern int NCZ_initialize(void);
 extern int NCZ_finalize(void);
