@@ -4,6 +4,7 @@
 #define NCLIST_H 1
 
 #include "ncexternl.h"
+#include <stddef.h>
 
 /* Define the type of the elements in the list*/
 
@@ -11,7 +12,7 @@
 extern "C" {
 #endif
 
-EXTERNL int nclistnull(void*);
+EXTERNL int nclistisnull(void*);
 
 typedef struct NClist {
   size_t alloc;
@@ -22,13 +23,14 @@ typedef struct NClist {
 EXTERNL NClist* nclistnew(void);
 EXTERNL int nclistfree(NClist*);
 EXTERNL int nclistfreeall(NClist*);
+EXTERNL int nclistclearall(NClist*);
 EXTERNL int nclistsetalloc(NClist*,size_t);
 EXTERNL int nclistsetlength(NClist*,size_t);
 
 /* Set the ith element; will overwrite previous contents; expand if needed */
 EXTERNL int nclistset(NClist*,size_t,void*);
 /* Get value at position i */
-EXTERNL void* nclistget(NClist*,size_t);/* Return the ith element of l */
+EXTERNL void* nclistget(const NClist*,size_t);/* Return the ith element of l */
 /* Insert at position i; will push up elements i..|seq|. */
 EXTERNL int nclistinsert(NClist*,size_t,void*);
 /* Remove element at position i; will move higher elements down */
@@ -52,10 +54,14 @@ EXTERNL int nclistelemremove(NClist* l, void* elem);
 EXTERNL int nclistunique(NClist*);
 
 /* Create a clone of a list; if deep, then assume it is a list of strings */
-EXTERNL NClist* nclistclone(NClist*, int deep);
+EXTERNL NClist* nclistclone(const NClist*, int deep);
 
+/* Extract the contents of a list, leaving list empty */
 EXTERNL void* nclistextract(NClist*);
 
+/* Append an uncounted NULL to the end of the list */
+EXTERNL int nclistnull(NClist*);
+
 /* Following are always "in-lined"*/
 #define nclistclear(l) nclistsetlength((l),0)
 #define nclistextend(l,len) nclistsetalloc((l),(len)+(l->alloc))
