@@ -1,12 +1,15 @@
 /*********************************************************************
- *   Copyright 1993, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *********************************************************************/
 #ifndef DEBUG_H
 #define DEBUG_H
 
+/* Warning: setting CATCHERROR has significant performance impact */
+#undef CATCHERROR
+
 #if 0
-#define DAPDEBUG 1
+#define DAPDEBUG 2
 #define OCDEBUG 1
 #endif
 
@@ -25,9 +28,10 @@
 #  endif
 #endif
 
+/* Dump info about constraint processing */
+#undef DCEVERBOSE
+
 #undef PARSEDEBUG
-/* Warning: setting CATCHERROR has significant performance impact */
-#define CATCHERROR
 
 #include <stdarg.h>
 #include <assert.h>
@@ -51,11 +55,12 @@ extern int dappanic(const char* fmt, ...);
 
 #ifdef CATCHERROR
 /* Place breakpoint on dapbreakpoint to catch errors close to where they occur*/
-#define THROW(e) dapthrow(e)
-#define THROWCHK(e) (void)dapthrow(e)
+/* Warning: do not evaluate more than once */
+#define THROW(e) dapthrow(e,__LINE__,__FILE__)
+#define THROWCHK(e) (void)dapthrow(e,__LINE__,__FILE__)
 
 extern int dapbreakpoint(int err);
-extern int dapthrow(int err);
+extern int dapthrow(int err, int lineno, const char* filename);
 #else
 #define THROW(e) (e)
 #define THROWCHK(e)
