@@ -15,6 +15,9 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
 
 #include <curl/curl.h>
 
@@ -39,6 +42,7 @@
 #include "ncuri.h"
 #include "nclog.h"
 #include "ncdap.h"
+#include "ncpathmgr.h"
 
 #include "d4util.h"
 
