@@ -10,8 +10,9 @@
 
 #include "config.h"
 #include "hdf5internal.h"
+#include "hdf5dispatch.h"
 
-#ifdef ENABLE_BYTERANGE
+#ifdef NETCDF_ENABLE_BYTERANGE
 #include "H5FDhttp.h"
 #endif
 
@@ -127,7 +128,7 @@ NC_HDF5_initialize(void)
     if (!nc4_hdf5_initialized)
         nc4_hdf5_initialize();
 
-#ifdef ENABLE_BYTERANGE
+#ifdef NETCDF_ENABLE_BYTERANGE
     (void)H5FD_http_init();
 #endif
     return NC4_provenance_init();
@@ -142,6 +143,9 @@ NC_HDF5_initialize(void)
 int
 NC_HDF5_finalize(void)
 {
+#ifdef NETCDF_ENABLE_BYTERANGE
+    (void)H5FD_http_finalize();
+#endif
     (void)nc4_hdf5_finalize();
     return NC_NOERR;
 }
