@@ -14,14 +14,11 @@
 #include "ncexternl.h"
 #include "nclog.h"
 
-#ifdef LOGGING
 #define ZLOG(tag,...) nclog(tag,__VA_ARGS__)
-#else
-#define ZLOG(tag,...)
-#endif
 
 #ifdef ZCATCH
 /* Place breakpoint on zbreakpoint to catch errors close to where they occur*/
+/* WARNING: Do not evaluate e more than once */
 #define THROW(e) zthrow((e),__FILE__, __func__, __LINE__)
 #define REPORT(e,msg) zreport((e),(msg),__FILE__, __func__, __LINE__)
 #define ZCHECK(e) if((e)) {THROW(stat); goto done;} else {}
@@ -81,4 +78,3 @@ struct ZUTEST {
 EXTERNL struct ZUTEST* zutest;
 
 #endif /*ZDEBUG_H*/
-
