@@ -14,6 +14,7 @@
 
 #include "zincludes.h"
 #include "zfilter.h"
+#include <stddef.h>
 
 /* Forward */
 static int NCZ_enddef(NC_FILE_INFO_T* h5);
@@ -103,7 +104,7 @@ static int
 NCZ_enddef(NC_FILE_INFO_T* h5)
 {
     NC_VAR_INFO_T *var;
-    int i,j;
+    size_t i,j;
     int stat = NC_NOERR;
 
     ZTRACE(1,"h5=%s",h5->hdr.name);
@@ -116,17 +117,6 @@ NCZ_enddef(NC_FILE_INFO_T* h5)
             assert(var);
             var->written_to = NC_TRUE; /* mark it written */
 	    var->created = 1;
-#if 0
-	    /* set the fill value and _FillValue attribute */
-	    if((stat = NCZ_ensure_fill_value(var))) goto done; /* ensure var->fill_value is set */
-            assert(var->no_fill || var->fill_value != NULL);
-	    /* rebuild the fill chunk */
-	    if((stat = NCZ_adjust_var_cache(var))) goto done;
-#ifdef ENABLE_NCZARR_FILTERS
-	    /* Build the filter working parameters for any filters */
-	    if((stat = NCZ_filter_setup(var))) goto done;
-#endif
-#endif /*0|1*/
         }
     }
     if((stat = ncz_enddef_netcdf4_file(h5))) goto done;
@@ -304,7 +294,7 @@ NCZ_inq(int ncid, int *ndimsp, int *nvarsp, int *nattsp, int *unlimdimidp)
     NC_FILE_INFO_T* file;
     NC_GRP_INFO_T *grp;
     int stat = NC_NOERR;
-    int i;
+    size_t i;
 
     LOG((2, "%s: ncid 0x%x", __func__, ncid));
 
@@ -377,13 +367,10 @@ ncz_sync_netcdf4_file(NC_FILE_INFO_T* file, int isclose)
     LOG((3, "%s", __func__));
     ZTRACE(2,"file=%s",file->hdr.name);
 
-    /* If we're in define mode, that's an error, for strict nc3 rules,
-     * otherwise, end define mode. */
+    /* End depend mode if needed. (Error checking for classic mode has
+     * already happened). */
     if (file->flags & NC_INDEF)
     {
-        if (file->cmode & NC_CLASSIC_MODEL)
-            return NC_EINDEFINE;
-
         /* Turn define mode off. */
         file->flags ^= NC_INDEF;
 
