@@ -22,7 +22,6 @@
 #define NCZ_CHUNKSIZE_FACTOR (10)
 #define NCZ_MIN_CHUNK_SIZE (2)
 
-
 /**************************************************/
 /* Constants */
 
@@ -39,56 +38,43 @@
 #  endif
 #endif
 
-/* V1 reserved objects */
-#define NCZMETAROOT "/.nczarr"
-#define NCZGROUP ".nczgroup"
-#define NCZARRAY ".nczarray"
-#define NCZATTRS ".nczattrs"
-/* Deprecated */
-#define NCZVARDEP ".nczvar"
-#define NCZATTRDEP ".nczattr"
-
 #define ZMETAROOT "/.zgroup"
+#define ZMETAATTR "/.zattrs"
 #define ZGROUP ".zgroup"
 #define ZATTRS ".zattrs"
 #define ZARRAY ".zarray"
 
-/* Pure Zarr pseudo names */
-#define ZDIMANON "_zdim"
-
 /* V2 Reserved Attributes */
 /*
-Inserted into /.zgroup
+For nczarr version 2.x.x, the following (key,value)
+pairs are stored in .zgroup and/or .zarray.
+
+Inserted into /.zattrs in root group
 _nczarr_superblock: {"version": "2.0.0"}
-Inserted into any .zgroup
+
+Inserted into any group level .zattrs 
 "_nczarr_group": "{
-\"dimensions\": {\"d1\": \"1\", \"d2\": \"1\",...}
-\"variables\": [\"v1\", \"v2\", ...]
+\"dimensions\": [{name: <dimname>, size: <integer>, unlimited: 1|0},...],
+\"arrays\": [\"v1\", \"v2\", ...]
 \"groups\": [\"g1\", \"g2\", ...]
 }"
-Inserted into any .zarray
+
+Inserted into any array level .zattrs
 "_nczarr_array": "{
-\"dimensions\": [\"/g1/g2/d1\", \"/d2\",...]
-\"storage\": \"scalar\"|\"contiguous\"|\"compact\"|\"chunked\"
+\"dimension_references\": [\"/g1/g2/d1\", \"/d2\",...]
+\"storage\": \"scalar\"|\"contiguous\"|\"chunked\"
 }"
-Inserted into any .zattrs ? or should it go into the container?
-"_nczarr_attrs": "{
+
+Inserted into any .zattrs
+"_nczarr_attr": "{
 \"types\": {\"attr1\": \"<i4\", \"attr2\": \"<i1\",...}
 }
-+
-+Note: _nczarr_attrs type include non-standard use of a zarr type "|U1" => NC_CHAR.
-+
 */
 
 #define NCZ_V2_SUPERBLOCK "_nczarr_superblock"
 #define NCZ_V2_GROUP   "_nczarr_group"
 #define NCZ_V2_ARRAY   "_nczarr_array"
-#define NCZ_V2_ATTR    NC_NCZARR_ATTR
-
-#define NCZ_V2_SUPERBLOCK_UC "_NCZARR_SUPERBLOCK"
-#define NCZ_V2_GROUP_UC   "_NCZARR_GROUP"
-#define NCZ_V2_ARRAY_UC   "_NCZARR_ARRAY"
-#define NCZ_V2_ATTR_UC    NC_NCZARR_ATTR_UC
+#define NCZ_V2_ATTR    "_nczarr_attr" /* Must match value in include/nc4internal.h */
 
 #define NCZARRCONTROL "nczarr"
 #define PUREZARRCONTROL "zarr"
@@ -96,17 +82,20 @@ Inserted into any .zattrs ? or should it go into the container?
 #define NOXARRAYCONTROL "noxarray"
 #define XARRAYSCALAR "_scalar_"
 
+#define NC_NCZARR_MAXSTRLEN_ATTR "_nczarr_maxstrlen"
+#define NC_NCZARR_DEFAULT_MAXSTRLEN_ATTR "_nczarr_default_maxstrlen"
+
 #define LEGAL_DIM_SEPARATORS "./"
 #define DFALT_DIM_SEPARATOR '.'
 
+#define islegaldimsep(c) ((c) != '\0' && strchr(LEGAL_DIM_SEPARATORS,(c)) != NULL)
+
 /* Default max string length for fixed length strings */
 #define NCZ_MAXSTR_DEFAULT 128
 
-#define islegaldimsep(c) ((c) != '\0' && strchr(LEGAL_DIM_SEPARATORS,(c)) != NULL)
-
 /* Mnemonics */
-#define ZCLEAR	0 /* For NCZ_copy_data */
-#define ZCLOSE	1 /* this is closeorabort as opposed to enddef */
+#define ZCLOSE	 1 /* this is closeorabort as opposed to enddef */
+#define ZREADING 1 /* this is reading data rather than writing */
 
 /* Useful macro */
 #define ncidforx(file,grpid) ((file)->controller->ext_ncid | (grpid))
@@ -142,16 +131,16 @@ typedef struct NCZ_FILE_INFO {
 	    unsigned long release;
 	} nczarr_version;
     } zarr;
-    int created; /* 1=> created 0=>open */
+    int creating; /* 1=> created 0=>open */
     int native_endianness; /* NC_ENDIAN_LITTLE | NC_ENDIAN_BIG */
-    char** envv_controls; /* Envv format */
+    NClist* controllist; /* Envv format */
     struct Controls {
         size64_t flags;
 #		define FLAG_PUREZARR    1
 #		define FLAG_SHOWFETCH   2
 #		define FLAG_LOGGING     4
 #		define FLAG_XARRAYDIMS  8
-#		define FLAG_NCZARR_V1   16
+#		define FLAG_NCZARR_KEY  16 /* _nczarr_xxx keys are stored in object and not in _nczarr_attrs */
 	NCZM_IMPL mapimpl;
     } controls;
     int default_maxstrlen; /* default max str size for variables of type string */
@@ -170,18 +159,13 @@ typedef struct  NCZ_ATT_INFO {
 /* Struct to hold ZARR-specific info for a group. */
 typedef struct NCZ_GRP_INFO {
     NCZcommon common;
-#if 0
-    /* The jcontent field stores the following:
-	1. List of (name,length) for dims in the group
-	2. List of (name,type) for user-defined types in the group
-	3. List of var names in the group
-	4. List of subgroups names in the group
-    */
-    NClist* dims;
-    NClist* types; /* currently not used */
-    NClist* vars;
-    NClist* grps;
-#endif
+    /* Read .zgroup and .zattrs once */
+    struct ZARROBJ {
+	char* prefix; /* prefix of .zgroup and .zattrs */
+	NCjson* obj; /* .zgroup|.zarray */
+	NCjson* atts;
+        int nczv1;   /* 1 => _nczarr_xxx are in obj and not attributes */
+    } zgroup;
 } NCZ_GRP_INFO_T;
 
 /* Struct to hold ZARR-specific info for a variable. */
@@ -196,6 +180,9 @@ typedef struct NCZ_VAR_INFO {
     char dimension_separator; /* '.' | '/' */
     NClist* incompletefilters;
     int maxstrlen; /* max length of strings for this variable */
+    /* Read .zarray and .zattrs once */
+    struct ZARROBJ zarray;
+    struct ZARROBJ zattrs;
 } NCZ_VAR_INFO_T;
 
 /* Struct to hold ZARR-specific info for a field. */
@@ -253,6 +240,7 @@ int ncz_closeorabort(NC_FILE_INFO_T*, void* params, int abort);
 
 /* zclose.c */
 int ncz_close_ncz_file(NC_FILE_INFO_T* file, int abort);
+int NCZ_zclose_var1(NC_VAR_INFO_T* var);
 
 /* zattr.c */
 int ncz_getattlist(NC_GRP_INFO_T *grp, int varid, NC_VAR_INFO_T **varp, NCindex **attlist);
