@@ -13,24 +13,7 @@
  */
 #include "nc4internal.h"
 #include "nc4dispatch.h"
-
-/** @internal Names of atomic types. */
-const char* nc4_atomic_name[NUM_ATOMIC_TYPES] = {"none", "byte", "char",
-                                           "short", "int", "float",
-                                           "double", "ubyte",
-                                           "ushort", "uint",
-                                           "int64", "uint64", "string"};
-
-/* The sizes of types may vary from platform to platform, but within
- * netCDF files, type sizes are fixed. */
-#define NC_CHAR_LEN sizeof(char)      /**< @internal Size of char. */
-#define NC_STRING_LEN sizeof(char *)  /**< @internal Size of char *. */
-#define NC_BYTE_LEN 1     /**< @internal Size of byte. */
-#define NC_SHORT_LEN 2    /**< @internal Size of short. */
-#define NC_INT_LEN 4      /**< @internal Size of int. */
-#define NC_FLOAT_LEN 4    /**< @internal Size of float. */
-#define NC_DOUBLE_LEN 8   /**< @internal Size of double. */
-#define NC_INT64_LEN 8    /**< @internal Size of int64. */
+#include <stddef.h>
 
 /**
  * @internal Find all user-defined types for a location. This finds
@@ -63,8 +46,7 @@ NC4_inq_typeids(int ncid, int *ntypes, int *typeids)
 
     /* Count types. */
     if (grp->type) {
-        int i;
-        for(i=0;i<ncindexsize(grp->type);i++)
+        for(size_t i=0;i<ncindexsize(grp->type);i++)
         {
             if((type = (NC_TYPE_INFO_T*)ncindexith(grp->type,i)) == NULL) continue;
             if (typeids)
@@ -81,8 +63,8 @@ NC4_inq_typeids(int ncid, int *ntypes, int *typeids)
 }
 
 /**
- * @internal Get the name and size of a type. For strings, 1 is
- * returned. For VLEN the base type len is returned.
+ * @internal Get the name and size of a type.
+ * For VLEN the base type len is returned.
  *
  * @param ncid File and group ID.
  * @param typeid1 Type ID.
@@ -99,10 +81,6 @@ NC4_inq_type(int ncid, nc_type typeid1, char *name, size_t *size)
 {
     NC_GRP_INFO_T *grp;
     NC_TYPE_INFO_T *type;
-    static const int atomic_size[NUM_ATOMIC_TYPES] = {0, NC_BYTE_LEN, NC_CHAR_LEN, NC_SHORT_LEN,
-                                                      NC_INT_LEN, NC_FLOAT_LEN, NC_DOUBLE_LEN,
-                                                      NC_BYTE_LEN, NC_SHORT_LEN, NC_INT_LEN, NC_INT64_LEN,
-                                                      NC_INT64_LEN, NC_STRING_LEN};
 
     int retval;
 
@@ -111,10 +89,7 @@ NC4_inq_type(int ncid, nc_type typeid1, char *name, size_t *size)
     /* If this is an atomic type, the answer is easy. */
     if (typeid1 < NUM_ATOMIC_TYPES)
     {
-        if (name)
-            strcpy(name, nc4_atomic_name[typeid1]);
-        if (size)
-            *size = atomic_size[typeid1];
+	if((retval=NC4_inq_atomic_type(typeid1, name, size))) return retval;
         return NC_NOERR;
     }
 
@@ -123,7 +98,7 @@ NC4_inq_type(int ncid, nc_type typeid1, char *name, size_t *size)
         return retval;
 
     /* Find this type. */
-    if (!(type = nclistget(grp->nc4_info->alltypes, typeid1)))
+    if (!(type = nclistget(grp->nc4_info->alltypes, (size_t)typeid1)))
         return NC_EBADTYPE;
 
     if (name)
@@ -133,8 +108,6 @@ NC4_inq_type(int ncid, nc_type typeid1, char *name, size_t *size)
     {
         if (type->nc_type_class == NC_VLEN)
             *size = sizeof(nc_vlen_t);
-        else if (type->nc_type_class == NC_STRING)
-            *size = 1;
         else
             *size = type->size;
     }
@@ -173,7 +146,7 @@ NC4_inq_user_type(int ncid, nc_type typeid1, char *name, size_t *size,
         return retval;
 
     /* Find this type. */
-    if (!(type = nclistget(grp->nc4_info->alltypes, typeid1)))
+    if (!(type = nclistget(grp->nc4_info->alltypes, (size_t)typeid1)))
         return NC_EBADTYPE;
 
     /* Count the number of fields. */
@@ -192,9 +165,9 @@ NC4_inq_user_type(int ncid, nc_type typeid1, char *name, size_t *size,
     {
         if (type->nc_type_class == NC_VLEN)
             *size = sizeof(nc_vlen_t);
-        else if (type->nc_type_class == NC_STRING)
-            *size = 1;
-        else
+        else if (type->nc_type_class == NC_STRING) {
+	    if((retval=NC4_inq_type(ncid,typeid1,NULL,size))) return retval;
+        } else
             *size = type->size;
     }
     if (name)
@@ -252,11 +225,11 @@ NC4_inq_compound_field(int ncid, nc_type typeid1, int fieldid, char *name,
         return retval;
 
     /* Find this type. */
-    if (!(type = nclistget(grp->nc4_info->alltypes, typeid1)))
+    if (!(type = nclistget(grp->nc4_info->alltypes, (size_t)typeid1)))
         return NC_EBADTYPE;
 
     /* Find the field. */
-    if (!(field = nclistget(type->u.c.field,fieldid)))
+    if (!(field = nclistget(type->u.c.field, (size_t)fieldid)))
         return NC_EBADFIELD;
 
     if (name)
@@ -296,7 +269,7 @@ NC4_inq_compound_fieldindex(int ncid, nc_type typeid1, const char *name, int *fi
     NC_FIELD_INFO_T *field;
     char norm_name[NC_MAX_NAME + 1];
     int retval;
-    int i;
+    size_t i;
 
     LOG((2, "nc_inq_compound_fieldindex: ncid 0x%x typeid %d name %s",
          ncid, typeid1, name));
@@ -336,8 +309,9 @@ NC4_inq_compound_fieldindex(int ncid, nc_type typeid1, const char *name, int *fi
 }
 
 /**
- * @internal Get enum name from enum value. Name size will be <=
- * NC_MAX_NAME.
+ * @internal Get enum name from enum value. Name size will be <= NC_MAX_NAME.
+ * If the value is not a legitimate enum identifier and the value is zero
+ * (the default HDF5 enum fill value), then return the identifier "_UNDEFINED".
  *
  * @param ncid File and group ID.
  * @param xtype Type ID.
@@ -347,7 +321,7 @@ NC4_inq_compound_fieldindex(int ncid, nc_type typeid1, const char *name, int *fi
  * @return ::NC_NOERR No error.
  * @return ::NC_EBADID Bad ncid.
  * @return ::NC_EBADTYPE Type not found.
- * @return ::NC_EINVAL Invalid type data.
+ * @return ::NC_EINVAL Invalid type data or no matching enum value is found
  * @author Ed Hartnett
  */
 int
@@ -357,7 +331,7 @@ NC4_inq_enum_ident(int ncid, nc_type xtype, long long value, char *identifier)
     NC_TYPE_INFO_T *type;
     NC_ENUM_MEMBER_INFO_T *enum_member;
     long long ll_val;
-    int i;
+    size_t i;
     int retval;
     int found;
 
@@ -368,7 +342,7 @@ NC4_inq_enum_ident(int ncid, nc_type xtype, long long value, char *identifier)
         return retval;
 
     /* Find this type. */
-    if (!(type = nclistget(grp->nc4_info->alltypes, xtype)))
+    if (!(type = nclistget(grp->nc4_info->alltypes, (size_t)xtype)))
         return NC_EBADTYPE;
 
     /* Complain if they are confused about the type. */
@@ -418,9 +392,13 @@ NC4_inq_enum_ident(int ncid, nc_type xtype, long long value, char *identifier)
     }
 
     /* If we didn't find it, life sucks for us. :-( */
-    if (!found)
-        return NC_EINVAL;
-
+    if(!found) {
+        if(value == 0) /* Special case for HDF5 default Fill Value*/
+	    strcpy(identifier, NC_UNDEFINED_ENUM_IDENT);
+        else
+            return NC_EINVAL;
+    }
+    
     return NC_NOERR;
 }
 
@@ -456,7 +434,7 @@ NC4_inq_enum_member(int ncid, nc_type typeid1, int idx, char *identifier,
         return retval;
 
     /* Find this type. */
-    if (!(type = nclistget(grp->nc4_info->alltypes, typeid1)))
+    if (!(type = nclistget(grp->nc4_info->alltypes, (size_t)typeid1)))
         return NC_EBADTYPE;
 
     /* Complain if they are confused about the type. */
@@ -464,7 +442,7 @@ NC4_inq_enum_member(int ncid, nc_type typeid1, int idx, char *identifier,
         return NC_EBADTYPE;
 
     /* Move to the desired enum member in the list. */
-    if (!(enum_member = nclistget(type->u.e.enum_member, idx)))
+    if (!(enum_member = nclistget(type->u.e.enum_member, (size_t)idx)))
         return NC_EINVAL;
 
     /* Give the people what they want. */
@@ -475,3 +453,276 @@ NC4_inq_enum_member(int ncid, nc_type typeid1, int idx, char *identifier,
 
     return NC_NOERR;
 }
+
+/**
+ * @internal Get the id of a type from the name.
+ *
+ * @param ncid File and group ID.
+ * @param name Name of type; might be fully qualified.
+ * @param typeidp Pointer that will get the type ID.
+ *
+ * @return ::NC_NOERR No error.
+ * @return ::NC_ENOMEM Out of memory.
+ * @return ::NC_EINVAL Bad size.
+ * @return ::NC_ENOTNC4 User types in netCDF-4 files only.
+ * @return ::NC_EBADTYPE Type not found.
+ * @author Ed Hartnett
+ */
+EXTERNL int
+NC4_inq_typeid(int ncid, const char *name, nc_type *typeidp)
+{
+    NC_GRP_INFO_T *grp;
+    NC_GRP_INFO_T *grptwo;
+    NC_FILE_INFO_T *h5;
+    NC_TYPE_INFO_T *type = NULL;
+    char *norm_name = NULL;
+    int i, retval = NC_NOERR;
+
+    /* Handle atomic types. */
+    for (i = 0; i < NUM_ATOMIC_TYPES; i++)
+        if (!strcmp(name, nc4_atomic_name[i]))
+        {
+            if (typeidp)
+                *typeidp = i;
+            goto done;
+        }
+
+    /* Find info for this file and group, and set pointer to each. */
+    if ((retval = nc4_find_grp_h5(ncid, &grp, &h5)))
+        goto done;
+    assert(h5 && grp);
+
+    /* If the first char is a /, this is a fully-qualified
+     * name. Otherwise, this had better be a local name (i.e. no / in
+     * the middle). */
+    if (name[0] != '/' && strstr(name, "/"))
+        {retval = NC_EINVAL; goto done;}
+
+    /* Normalize name. */
+    if (!(norm_name = (char*)malloc(strlen(name) + 1)))
+        {retval = NC_ENOMEM; goto done;}
+    if ((retval = nc4_normalize_name(name, norm_name)))
+	goto done;
+
+    /* If this is a fqn, then walk the sequence of parent groups to the last group
+       and see if that group has a type of the right name */
+    if(name[0] == '/') { /* FQN */
+	int rootncid = (grp->nc4_info->root_grp->hdr.id | grp->nc4_info->controller->ext_ncid);
+	int parent = 0;
+	char* lastname = strrchr(norm_name,'/'); /* break off the last segment: the type name */
+	if(lastname == norm_name)
+	    {retval = NC_EINVAL; goto done;}
+	*lastname++ = '\0'; /* break off the lastsegment */
+	if((retval = NC4_inq_grp_full_ncid(rootncid,norm_name,&parent))) 
+	    goto done;
+	/* Get parent info */
+	if((retval=nc4_find_nc4_grp(parent,&grp)))
+	    goto done;
+	/* See if type exists in this group */
+        type = (NC_TYPE_INFO_T*)ncindexlookup(grp->type,lastname);
+	if(type == NULL) 	
+	    {retval = NC_EBADTYPE; goto done;}
+	goto done;
+    }
+
+    /* Is the type in this group? If not, search parents. */
+    for (grptwo = grp; grptwo; grptwo = grptwo->parent) {
+        type = (NC_TYPE_INFO_T*)ncindexlookup(grptwo->type,norm_name);
+        if(type)
+        {
+            if (typeidp)
+                *typeidp = type->hdr.id;
+            break;
+        }
+    }
+
+    /* Still didn't find type? Search file recursively, starting at the
+     * root group. */
+    if (!type)
+        if ((type = nc4_rec_find_named_type(grp->nc4_info->root_grp, norm_name)))
+            if (typeidp)
+                *typeidp = type->hdr.id;
+
+    /* OK, I give up already! */
+    if (!type)
+        {retval = NC_EBADTYPE; goto done;}
+
+done:
+    nullfree(norm_name);
+    return retval;
+}
+
+/**
+ * @internal Get the class of a type
+ *
+ * @param h5 Pointer to the HDF5 file info struct.
+ * @param xtype NetCDF type ID.
+ * @param type_class Pointer that gets class of type, NC_INT,
+ * NC_FLOAT, NC_CHAR, or NC_STRING, NC_ENUM, NC_VLEN, NC_COMPOUND, or
+ * NC_OPAQUE.
+ *
+ * @return ::NC_NOERR No error.
+ * @author Ed Hartnett
+ */
+int
+nc4_get_typeclass(const NC_FILE_INFO_T *h5, nc_type xtype, int *type_class)
+{
+    int retval = NC_NOERR;
+
+    LOG((4, "%s xtype: %d", __func__, xtype));
+    assert(type_class);
+
+    /* If this is an atomic type, the answer is easy. */
+    if (xtype <= NC_STRING)
+    {
+        switch (xtype)
+        {
+        case NC_BYTE:
+        case NC_UBYTE:
+        case NC_SHORT:
+        case NC_USHORT:
+        case NC_INT:
+        case NC_UINT:
+        case NC_INT64:
+        case NC_UINT64:
+            /* NC_INT is class used for all integral types */
+            *type_class = NC_INT;
+            break;
+
+        case NC_FLOAT:
+        case NC_DOUBLE:
+            /* NC_FLOAT is class used for all floating-point types */
+            *type_class = NC_FLOAT;
+            break;
+
+        case NC_CHAR:
+            *type_class = NC_CHAR;
+            break;
+
+        case NC_STRING:
+            *type_class = NC_STRING;
+            break;
+
+        default:
+            BAIL(NC_EBADTYPE);
+        }
+    }
+    else
+    {
+        NC_TYPE_INFO_T *type;
+
+        /* See if it's a used-defined type */
+        if ((retval = nc4_find_type(h5, xtype, &type)))
+            BAIL(retval);
+        if (!type)
+            BAIL(NC_EBADTYPE);
+
+        *type_class = type->nc_type_class;
+    }
+
+exit:
+    return retval;
+}
+
+/**
+ * @internal return 1 if type is fixed size; 0 otherwise.
+ *
+ * @param ncid file id
+ * @param xtype type id
+ * @param fixedsizep pointer into which 1/0 is stored
+ *
+ * @return ::NC_NOERR
+ * @return ::NC_EBADTYPE if bad type
+ * @author Dennis Heimbigner
+ */
+int
+NC4_inq_type_fixed_size(int ncid, nc_type xtype, int* fixedsizep)
+{
+    int stat = NC_NOERR;   
+    int f = 0;
+    NC_FILE_INFO_T* h5 = NULL;
+    NC_TYPE_INFO_T* typ = NULL;
+
+    if(xtype < NC_STRING) {f = 1; goto done;}
+    if(xtype == NC_STRING) {f = 0; goto done;}
+
+#ifdef USE_NETCDF4
+    /* Must be user type */
+    if ((stat = nc4_find_grp_h5(ncid, NULL, &h5)))
+        goto done;
+    if((stat = nc4_find_type(h5,xtype,&typ))) goto done;
+    f = !typ->varsized;
+#endif
+done:
+    if(fixedsizep) *fixedsizep = f;
+    return stat;
+}
+
+
+/**
+For types with one or more subtypes (e.g. basetype or
+fieldtype), determine the varsizedness of the type based on the
+basetype.  The idea is to inform the code of the fact that
+parenttype has addedtype "inserted" into it.
+@param parenttype
+@param subtype
+*/
+
+int
+NC4_recheck_varsize(NC_TYPE_INFO_T* parenttype, nc_type subtype)
+{
+    int stat = NC_NOERR;
+    NC_FILE_INFO_T* file = NULL;
+    NC_TYPE_INFO_T* utype = NULL;
+    if(subtype < NC_STRING) goto done; /* will not change the "variable-sizedness" of parenttype */
+    if(subtype == NC_STRING) {parenttype->varsized = 1; goto done;}
+    /* Get the inferred user-type */
+    file = parenttype->container->nc4_info;
+    if((stat = nc4_find_type(file,subtype,&utype))) goto done;
+    switch (utype->nc_type_class) {
+    case NC_OPAQUE: case NC_ENUM: break; /* no change */
+    case NC_VLEN: parenttype->varsized = 1; break;
+    case NC_COMPOUND: if(utype->varsized) parenttype->varsized = 1; break;
+    }
+    
+done:
+    return stat;
+}
+
+/**
+When creating a type, mark it as variable-sized if known for sure.
+@param typ
+*/
+
+int
+NC4_set_varsize(NC_TYPE_INFO_T* typ)
+{
+    int stat = NC_NOERR;
+    if(typ->hdr.id < NC_STRING) goto done; /* will not change the "variable-sizedness" of typ */
+    if(typ->hdr.id == NC_STRING) {typ->varsized = 1; goto done;}
+    switch (typ->nc_type_class) {
+    case NC_OPAQUE: case NC_ENUM: break; /* no change */
+    case NC_VLEN: typ->varsized = 1; break;
+    case NC_COMPOUND: typ->varsized = 0; break; /* until proven otherwise */
+    }
+done:
+    return stat;
+}
+
+/**
+ * Test if a variable's type is fixed sized or not.
+ * @param var - to test
+ * @return 0 if fixed size, 1 otherwise.
+ */
+int
+NC4_var_varsized(NC_VAR_INFO_T* var)
+{
+    NC_TYPE_INFO_T* vtype = NULL;
+    
+    /* Check the variable type */
+    vtype = var->type_info;
+    if(vtype->hdr.id < NC_STRING) return 0;
+    if(vtype->hdr.id == NC_STRING) return 1;
+    return vtype->varsized;
+}
+
