@@ -1,5 +1,5 @@
 /*
-  Copyright 2008, UCAR/Unidata
+  Copyright 2018, UCAR/Unidata
   See COPYRIGHT file for copying and redistribution conditions.
 
   This program tests the fix for a large file bug in versions previous
@@ -22,7 +22,7 @@
 
 /* Test with both classic and 64-bit offset files. If netcdf-4 is
  * included, test with both netCDF-4 format variants also. */
-#ifdef USE_NETCDF4
+#ifdef USE_HDF5
 #define NUM_FORMATS (4)
 #else
 #define NUM_FORMATS (2)
@@ -45,9 +45,6 @@ static int
 test_big_var(const char *testfile)
 {
     int ncid, varid, dimids[NUMDIMS];
-    size_t index[NUMDIMS];
-    int nval = 99;
-    int nval_in;
     size_t start[NUMDIMS] = {0, 0, 0};
     size_t count[NUMDIMS] = {1, DIM1, DIM2};
     signed char data[DIM1][DIM2];
@@ -92,7 +89,7 @@ test_big_var(const char *testfile)
 	{
 	    if (data[i][j] != 42 )
 	    {
-		printf("error on start[0]: %d i: %d j: %d expected %d got %d\n",
+		printf("error on start[0]: %ld i: %d j: %d expected %d got %d\n",
 		       start[0], i, j, 42, data[i][j]);
 		ERR;
 		if(nerrs++ > 1)
@@ -106,7 +103,7 @@ test_big_var(const char *testfile)
 	{
 	    if (data[i][j] != 19 )
 	    {
-	      printf("error on start[0]: %d i: %d j: %d expected %d got %d\n",
+	      printf("error on start[0]: %ld i: %d j: %d expected %d got %d\n",
 		     start[0], i, j, 19, data[i][j]);
 	      ERR;
 		if(nerrs++ > 1)
@@ -124,7 +121,7 @@ main(int argc, char **argv) {
     char testfile[NC_MAX_NAME + 1];
 
     printf("\n*** Testing multidimensional variable with more than 2**32 values\n");
-    sprintf(testfile, "%s/%s", TEMP_LARGE, FILE_NAME);
+    snprintf(testfile, sizeof(testfile), "%s/%s", TEMP_LARGE, FILE_NAME);
     for (i = NC_FORMAT_CLASSIC; i <= NUM_FORMATS; i++)
     {
        printf("*** testing format %d file with byte variable with > 2**32 values...", i);
