@@ -10,6 +10,7 @@
 
 #include "netcdf.h"
 #include "netcdf_filter.h"
+#include "netcdf_aux.h"
 
 #undef DEBUG
 
@@ -122,6 +123,11 @@ static const char* spectype[] = {"i", "b", "ub", "s", "us", "i", "ui", "i", "i",
 
 static int nerrs = 0;
 
+#ifdef WORDS_BIGENDIAN
+static void byteswap8(unsigned char* mem);
+static void byteswap4(unsigned char* mem);
+#endif
+
 static void
 mismatch(size_t i, unsigned int *params, const char* tag)
 {
@@ -168,14 +174,14 @@ buildbaseline(void)
     insert(5,&val4,sizeof(val4)); /* unsigned int*/
     val4 = 2147483647; /*0x7fffffff*/
     insert(6,&val4,sizeof(val4)); /* signed int */
-    val4 = (-2147483647)-1; /*0x80000000*/
-    insert(7,&val4,sizeof(val4)); /* signed int */
+    int sval4 = (-2147483647)-1; /*0x80000000*/
+    insert(7,&sval4,sizeof(sval4)); /* signed int */
     val4 = 4294967295U; /*0xffffffff*/
     insert(8,&val4,sizeof(val4)); /* unsigned int */
     float4 = (float)FLTVAL;
     insert(9,&float4,sizeof(float4)); /*float */
-    val8 = LONGLONGVAL;
-    insert(10,&val8,sizeof(val8)); /* signed long long */
+    long long sval8 = LONGLONGVAL;
+    insert(10,&sval8,sizeof(sval8)); /* signed long long */
     val8 = ULONGLONGVAL;
     insert(12,&val8,sizeof(val8)); /* unsigned long long */
     float8 = DBLVAL;
@@ -187,26 +193,30 @@ int
 main(int argc, char **argv)
 {
     int stat = 0;
-    unsigned int id = 0;
-    size_t i,nparams = 0;
-    unsigned int* params = NULL;
+    size_t i;
     /* Provide for 8-byte values */
     long long basell;
     unsigned long long baseull;
     float basef;
     double based;
+    unsigned int filterid = 0;
+    size_t nparams;
+    unsigned int* params = NULL;
 
     printf("\nTesting filter parser.\n");
 
     buildbaseline(); /* Build our comparison vector */
 
-    stat = NC_parsefilterspec(spec,&id,&nparams,&params);
+    stat = ncaux_h5filterspec_parse(spec,&filterid,&nparams,&params);
     if(stat) {
-	fprintf(stderr,"NC_parsefilterspec failed\n");
-	exit(1);
+	fprintf(stderr,"ncaux_filter_parsespec failed\n");
+	goto done;
+    }
+    
+    if(filterid != PARAMS_ID) {
+        fprintf(stderr,"mismatch: id: expected=%u actual=%u\n",(unsigned int)PARAMS_ID,filterid);
+	nerrs++;
     }
-    if(id != PARAMS_ID)
-        fprintf(stderr,"mismatch: id: expected=%u actual=%u\n",PARAMS_ID,id);
 
     /* Do all the 32 bit tests */
     for(i=0;i<=8;i++) {
@@ -223,7 +233,7 @@ main(int argc, char **argv)
     /* signed long long */
     ul.ui[0] = params[10];
     ul.ui[1] = params[11];
-    NC_filterfix8((unsigned char*)&ul.ll,1);
+    ncaux_h5filterspec_fix8((unsigned char*)&ul.ll,1);
     memcpy(&basell,&baseline[10],8);
     if(ul.ll != basell)
 	mismatch2(10,params,"ul.ll");
@@ -231,7 +241,7 @@ main(int argc, char **argv)
     /* unsigned long long */
     ul.ui[0] = params[12];
     ul.ui[1] = params[13];
-    NC_filterfix8((unsigned char*)&ul.ull,1);
+    ncaux_h5filterspec_fix8((unsigned char*)&ul.ull,1);
     memcpy(&baseull,&baseline[12],8);
     if(ul.ull != baseull)
 	mismatch2(12,params,"ul.ull");
@@ -239,7 +249,7 @@ main(int argc, char **argv)
     /* double */
     ud.ui[0] = params[14];
     ud.ui[1] = params[15];
-    NC_filterfix8((unsigned char*)&ud.d,1);
+    ncaux_h5filterspec_fix8((unsigned char*)&ud.d,1);
     memcpy(&based,&baseline[14],8);
     if(ud.d != based)
 	mismatch2(14,params,"ud.d");
@@ -247,8 +257,9 @@ main(int argc, char **argv)
     if (!nerrs)
        printf("SUCCESS!!\n");
 
+done:
     if(params) free(params);
-    return (nerrs > 0 ? 1 : 0);
+    return (stat || nerrs > 0 ? 1 : 0);
 }
 
 #ifdef DEBUG
@@ -284,151 +295,6 @@ gettype(const int q0, const int q1, int* isunsignedp)
     return type;
 }
 
-static int
-parsefilterspec(const char* spec, unsigned int* idp, size_t* nparamsp, unsigned int** paramsp)
-{
-    int stat = NC_NOERR;
-    int sstat; /* for scanf */
-    char* p;
-    char* sdata = NULL;
-    unsigned int id;
-    size_t count; /* no. of comma delimited params */
-    size_t nparams; /* final no. of unsigned ints */
-    size_t len;
-    int i;
-    unsigned int* ulist = NULL;
-    unsigned char mem[8];
-
-    if(spec == NULL || strlen(spec) == 0) goto fail;
-    sdata = strdup(spec);
-
-    /* Count number of parameters + id and delimit */
-    p=sdata;
-    for(count=0;;count++) {
-        char* q = strchr(p,',');
-	if(q == NULL) break;
-	*q++ = '\0';
-	p = q;
-    }
-    count++; /* for final piece */
-
-    if(count == 0)
-	goto fail; /* no id and no parameters */
-
-    /* Extract the filter id */
-    p = sdata;
-    sstat = sscanf(p,"%u",&id);
-    if(sstat != 1) goto fail;
-    /* skip past the filter id */
-    p = p + strlen(p) + 1;
-    count--;
-
-    /* Allocate the max needed space; *2 in case the params are all doubles */
-    ulist = (unsigned int*)malloc(sizeof(unsigned int)*(count)*2);
-    if(ulist == NULL) goto fail;
-
-    /* walk and convert */
-    nparams = 0; /* actual count */
-    for(i=0;i<count;i++) { /* step thru param strings */
-	unsigned long long val64u;
-	unsigned int val32u;
-	double vald;
-	float valf;
-	unsigned int *vector;
-	int isunsigned = 0;
-	int isnegative = 0;
-	int type = 0;
-	char* q;
-
-	len = strlen(p);
-	/* skip leading white space */
-	while(strchr(" 	",*p) != NULL) {p++; len--;}
-	/* Get leading sign character, if any */
-	if(*p == '-') isnegative = 1;
-        /* Get trailing type tag characters */
-	switch (len) {
-	case 0:
-	    goto fail; /* empty parameter */
-	case 1:
-	case 2:
-	    q = (p + len) - 1; /* point to last char */
-	    type = gettype(*q,'\0',&isunsigned);
-	    break;
-	default: /* > 2 => we might have a two letter tag */
-	    q = (p + len) - 2;
-	    type = gettype(*q,*(q+1),&isunsigned);
-	    break;
-	}
-
-	/* Now parse */
-	switch (type) {
-	case 'b':
-	case 's':
-	case 'i':
- 	    /* special case for a positive integer;for back compatibility.*/
-	    if(!isnegative)
-	        sstat = sscanf(p,"%u",&val32u);
-	    else
-                sstat = sscanf(p,"%d",(int*)&val32u);
-	    if(sstat != 1) goto fail;
-	    switch(type) {
-	    case 'b': val32u = (val32u & 0xFF); break;
-	    case 's': val32u = (val32u & 0xFFFF); break;
-	    }
-	    ulist[nparams++] = val32u;
-	    break;
-
-	case 'f':
-	    sstat = sscanf(p,"%lf",&vald);
-	    if(sstat != 1) goto fail;
-	    valf = (float)vald;
-	    ulist[nparams++] = *(unsigned int*)&valf;
-	    break;
-
-	/* The following are 8-byte values, so we must swap pieces if this
-           is a little endian machine */	
-	case 'd':
-	    sstat = sscanf(p,"%lf",&vald);
-	    if(sstat != 1) goto fail;
-	    memcpy(mem,&vald,sizeof(mem));
-	    NC_filterfix8(mem,0);
-	    vector = (unsigned int*)mem;
-	    ulist[nparams++] = vector[0];
-	    ulist[nparams++] = vector[1];
-	    break;
-	case 'l': /* long long */
-	    if(isunsigned)
-	        sstat = sscanf(p,"%llu",&val64u);
-	    else
-                sstat = sscanf(p,"%lld",(long long*)&val64u);
-	    if(sstat != 1) goto fail;
-	    memcpy(mem,&val64u,sizeof(mem));
-	    NC_filterfix8(mem,0);
-	    vector = (unsigned int*)&mem;
-	    ulist[nparams++] = vector[0];
-	    ulist[nparams++] = vector[1];
-	    break;
-	default:
-	    goto fail;
-	}
-        p = p + strlen(p) + 1; /* move to next param */
-    }
-    /* Now return results */
-    if(idp) *idp = id;
-    if(nparamsp) *nparamsp = nparams;
-    if(paramsp) {
-       *paramsp = ulist;
-       ulist = NULL; /* avoid duplicate free */
-    }
-done:
-    if(sdata) free(sdata);
-    if(ulist) free(ulist);
-    return stat;
-fail:
-    stat = NC_EFILTER;
-    goto done;
-}
-
 #ifdef WORDS_BIGENDIAN
 /* Byte swap an 8-byte integer in place */
 static void
@@ -464,8 +330,9 @@ byteswap4(unsigned char* mem)
 
 #endif
 
+#if 0
 static void
-NC_filterfix8(unsigned char* mem, int decode)
+NC4_h5filterspec_fix8(unsigned char* mem, int decode)
 {
 #ifdef WORDS_BIGENDIAN
     if(decode) { /* Apply inverse of the encode case */
@@ -481,6 +348,7 @@ NC_filterfix8(unsigned char* mem, int decode)
     /* No action is necessary */
 #endif	    
 }
+#endif
 
 #endif /*DEBUG*/
 
