@@ -1,3 +1,14 @@
+/*! \file
+
+Copyright 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
+2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
+2015, 2016, 2017, 2018
+University Corporation for Atmospheric Research/Unidata.
+
+See \ref copyright file for more info.
+
+*/
+
 /*
  * Test renames of vars and dims. It's a surprisingly tricky business.
  *
@@ -22,6 +33,13 @@
 #define VAR_RANK 1              /* all vars in this test are of same rank */
 #define DIM_LEN 2               /* all dims in this test are of same len */
 
+/* For the tests based on Charlie Zender's rename bug test. */
+#define CHARLIE_TEST_FILE "tst_charlie_rename_coord_dim.nc"
+#define LON "lon"
+#define LONGITUDE "longitude"
+#define DIM1_LEN 4
+#define NDIM1 1
+
 /* Test data. */
 int lats[DIM_LEN] = {-90, 90};
 float rh[DIM_LEN] = {0.25, 0.75};
@@ -61,7 +79,7 @@ check_file(int ncid, char *var0_name, char *var1_name, char *dim_name)
 
    /* printf("checking for vars %s and %s, dim %s\n", var0_name, var1_name, */
    /*        dim_name); */
-   
+
    /* Check vars. Ids will change because of rename. */
    if (nc_inq_varid(ncid, var0_name, &varid)) ERR;
    if (nc_inq_varid(ncid, var1_name, &var2id)) ERR;
@@ -69,7 +87,7 @@ check_file(int ncid, char *var0_name, char *var1_name, char *dim_name)
    /* Check dim. */
    if (nc_inq_dimid(ncid, dim_name, &dimid)) ERR;
    if (dimid != 0) ERR;
-   
+
    /* Check the lats. */
    if (nc_get_var_int(ncid, varid, lats_in)) ERR;
    for (ii = 0; ii < DIM_LEN; ii++)
@@ -92,7 +110,7 @@ check_charlies_file(char *file, char *dim_name, char *var_name)
 {
    int ncid;
    int varid, dimid;
-   
+
    if (nc_open(file, 0, &ncid)) ERR;
    if (nc_inq_varid(ncid, var_name, &varid)) ERR;
    if (nc_inq_dimid(ncid, dim_name, &dimid)) ERR;
@@ -101,6 +119,19 @@ check_charlies_file(char *file, char *dim_name, char *var_name)
    return NC_NOERR;
 }
 
+/* Check a data-free version of the already-open file created in a
+ * test. */
+int
+check_charlies_no_enddef_file(int ncid, char *dim_name, char *var_name)
+{
+   int varid, dimid;
+
+   if (nc_inq_varid(ncid, var_name, &varid)) ERR;
+   if (nc_inq_dimid(ncid, dim_name, &dimid)) ERR;
+   if (varid || dimid) ERR;
+   return NC_NOERR;
+}
+
 int
 main(int argc, char **argv)
 {
@@ -110,11 +141,10 @@ main(int argc, char **argv)
    char *file_names[] = {FILE_NAME3, FILE_NAME4};
    int format;
 
-   fprintf(stderr,"*** Testing netcdf rename bugs and fixes.\n");
-   nc_set_log_level(5);
+   printf("*** Testing netcdf rename bugs and fixes.\n");
+   /* nc_set_log_level(5); */
 
    for (format = 0; format < NUM_FORMATS; format++)
-   /* for (format = 0; format < 1; format++) */
    {
       int ncid, dimid, varid, var2id;
       int lats[DIM_LEN] = {-90, 90};
@@ -123,13 +153,83 @@ main(int argc, char **argv)
       float rh_in[DIM_LEN];
       int ii;
 
-      fprintf(stderr,"*** Test Charlie's test for renaming...");
+      printf("*** Test Charlie's test for renaming without enddef...");
+      {
+         int ncid, dimid, varid;
+
+         /* Create a nice, simple file. This file will contain one
+          * dataset, "lon", which is a dimscale. */
+         if (nc_create(CHARLIE_TEST_FILE, NC_NETCDF4, &ncid)) ERR;
+         if (nc_def_dim(ncid, LON, DIM1_LEN, &dimid)) ERR;
+         if (nc_def_var(ncid, LON, NC_FLOAT, NDIM1, &dimid, &varid)) ERR;
+
+         /* Check the file. */
+         if (check_charlies_no_enddef_file(ncid, LON, LON)) ERR;
+
+         /* Rename the dimension. This will cause lon to stop being a
+          * coord var and dimscale, and prepare to create a new
+          * dimscale without var dataset "longitude". Dataset "lon"
+          * will point to "longitude" as its dimscale. */
+         if (nc_rename_dim(ncid, 0, LONGITUDE)) ERR;
+         if (check_charlies_no_enddef_file(ncid, LONGITUDE, LON)) ERR;
+
+         /* Rename the variable. This will remove the (as yet
+          * unwritten) dimscale-only dataset "longitude" and rename
+          * the extisting dataset "lon" to "longitude". Variable
+          * "longitude" will become a coordinate var. */
+         if (nc_rename_var(ncid, 0, LONGITUDE)) ERR;
+         if (check_charlies_no_enddef_file(ncid, LONGITUDE, LONGITUDE)) ERR;
+         if (nc_close(ncid)) ERR;
+
+         /* Reopen the file and check. */
+         if (check_charlies_file(CHARLIE_TEST_FILE, LONGITUDE, LONGITUDE)) ERR;
+
+      }
+      SUMMARIZE_ERR;
+      printf("*** Test Charlie's test for renaming with one enddef...");
+      {
+         int ncid, dimid, varid;
+
+#ifdef DEBUG
+         nc_set_log_level(5);
+#endif
+
+         /* Create a nice, simple file. This file will contain one
+          * dataset, "lon", which is a dimscale. */
+         if (nc_create(CHARLIE_TEST_FILE, NC_NETCDF4, &ncid)) ERR;
+         if (nc_def_dim(ncid, LON, DIM1_LEN, &dimid)) ERR;
+         if (nc_def_var(ncid, LON, NC_FLOAT, NDIM1, &dimid, &varid)) ERR;
+
+         /* Check the file. */
+         if (check_charlies_no_enddef_file(ncid, LON, LON)) ERR;
+
+         /* Rename the dimension. This will cause lon to stop being a
+          * coord var and dimscale, and prepare to create a new
+          * dimscale without var dataset "longitude". Dataset "lon"
+          * will point to "longitude" as its dimscale. */
+         if (nc_rename_dim(ncid, 0, LONGITUDE)) ERR;
+         if (check_charlies_no_enddef_file(ncid, LONGITUDE, LON)) ERR;
+
+         /* Trigger write to disk. */
+         if (nc_enddef(ncid)) ERR;
+         if (nc_redef(ncid)) ERR;
+         if (check_charlies_no_enddef_file(ncid, LONGITUDE, LON)) ERR;
+
+         /* Rename the variable. This will remove the (as yet
+          * unwritten) dimscale-only dataset "longitude" and rename
+          * the extisting dataset "lon" to "longitude". Variable
+          * "longitude" will become a coordinate var. */
+         if (nc_rename_var(ncid, 0, LONGITUDE)) ERR;
+         if (check_charlies_no_enddef_file(ncid, LONGITUDE, LONGITUDE)) ERR;
+         if (nc_close(ncid)) ERR;
+
+         /* Reopen the file and check. */
+         if (check_charlies_file(CHARLIE_TEST_FILE, LONGITUDE, LONGITUDE)) ERR;
+
+      }
+      SUMMARIZE_ERR;
+      printf("*** Test Charlie's test for renaming with enddef...");
       {
-#define CHARLIE_TEST_FILE "tst_charlie_rename_coord_dim.nc"
-#define LON "lon"
-#define LONGITUDE "longitude"
-#define DIM1_LEN 4
-#define NDIM1 1
          int ncid, dimid, varid;
          float data[DIM1_LEN] = {0, 90.0, 180.0, 270.0};
 
@@ -162,14 +262,14 @@ main(int argc, char **argv)
           * the dimscale-only dataset "longitude" and rename the
           * extisting dataset "lon" to "longitude". Variable
           * "longitude" will become a coordinate var. */
-         /* if (nc_open(CHARLIE_TEST_FILE, NC_WRITE, &ncid)) ERR; */
-         /* if (nc_redef(ncid)) ERR; */
-         /* if (nc_rename_var(ncid, 0, LONGITUDE)) ERR; */
-         /* if (nc_enddef(ncid)) ERR; */
-         /* if (nc_close(ncid)) ERR; */
+         if (nc_open(CHARLIE_TEST_FILE, NC_WRITE, &ncid)) ERR;
+         if (nc_redef(ncid)) ERR;
+         if (nc_rename_var(ncid, 0, LONGITUDE)) ERR;
+         if (nc_enddef(ncid)) ERR;
+         if (nc_close(ncid)) ERR;
 
          /* Reopen the file to check. */
-         /* if (check_charlies_file(CHARLIE_TEST_FILE, LONGITUDE, LONGITUDE)) ERR; */
+         if (check_charlies_file(CHARLIE_TEST_FILE, LONGITUDE, LONGITUDE)) ERR;
       }
       SUMMARIZE_ERR;
 
@@ -177,7 +277,7 @@ main(int argc, char **argv)
       {
          int ncid, varid, var2id;
          int dimid;
-         
+
          if (nc_set_default_format(formats[format], NULL)) ERR;
          if (nc_create(file_names[format], 0, &ncid)) ERR;
          if (nc_def_dim(ncid, LAT, DIM_LEN, &dimid)) ERR;
@@ -187,7 +287,7 @@ main(int argc, char **argv)
          /* Now rename the dim. */
          if (nc_rename_dim(ncid, dimid, TAL)) ERR;
          if (nc_rename_var(ncid, varid, TAL)) ERR;
-         
+
          if (nc_enddef(ncid)) ERR;    /* not necessary for netCDF-4 files */
          if (nc_put_var_int(ncid, varid, lats)) ERR;
          if (nc_put_var_float(ncid, var2id, rh)) ERR;
@@ -204,7 +304,7 @@ main(int argc, char **argv)
       {
          int ncid, varid, var2id;
          int dimid;
-         
+
          if (nc_set_default_format(formats[format], NULL)) ERR;
          if (nc_create(file_names[format], 0, &ncid)) ERR;
          if (nc_def_dim(ncid, LAT, DIM_LEN, &dimid)) ERR;
@@ -216,7 +316,7 @@ main(int argc, char **argv)
          if (nc_rename_var(ncid, varid, TAL2)) ERR;
          if (nc_rename_dim(ncid, dimid, TAL)) ERR;
          if (nc_enddef(ncid)) ERR;
-         
+
          if (nc_put_var_int(ncid, varid, lats)) ERR;
          if (nc_put_var_float(ncid, var2id, rh)) ERR;
          if (nc_close(ncid)) ERR;
@@ -231,7 +331,7 @@ main(int argc, char **argv)
       {
          int ncid, varid, var2id;
          int dimid;
-         
+
          /* This will create a HDF5 file with two datasets, RH, and
           * LAT. LAT is a dimscale. RH points to dimscale LAT. Life is
           * so simple. */
@@ -278,7 +378,7 @@ main(int argc, char **argv)
       {
          int ncid, varid, var2id;
          int dimid;
-         
+
          if (nc_set_default_format(formats[format], NULL)) ERR;
          if (nc_create(file_names[format], 0, &ncid)) ERR;
          if (nc_def_dim(ncid, LAT, DIM_LEN, &dimid)) ERR;
@@ -291,19 +391,18 @@ main(int argc, char **argv)
          if (nc_rename_dim(ncid, dimid, TAL)) ERR;
          if (nc_rename_var(ncid, varid, TAL)) ERR;
          if (nc_enddef(ncid)) ERR;    /* not necessary for netCDF-4 files */
-         
+
          if (nc_put_var_int(ncid, varid, lats)) ERR;
          if (nc_put_var_float(ncid, var2id, rh)) ERR;
          if (nc_close(ncid)) ERR;
 
          /* Reopen and check. */
          if (nc_open(file_names[format], NC_WRITE, &ncid)) ERR;
-         /* if (check_file(ncid, LAT, RH, TAL)) ERR; */
-         /* if (check_file(ncid, TAL, RH, TAL)) ERR; */
+         if (check_file(ncid, TAL, RH, TAL)) ERR;
          if (nc_close(ncid)) ERR;
       }
       SUMMARIZE_ERR;
-      
+
       printf("*** testing renaming after enddef for %s...", fmt_names[format]);
       {
          /* Create a file with datasets LAT, RH. LAT is a dimscale. RH
@@ -327,25 +426,27 @@ main(int argc, char **argv)
           * TAL. LAT will become a dimscale. RH will point to LAT. */
          if (nc_rename_var(ncid, varid, TAL)) ERR;
          if (nc_enddef(ncid)) ERR;
+         /* This should work but does not. It is a known rename
+          * bug. Rename is coming! */
          /* if (check_file(ncid, LAT, RH, TAL)) ERR; */
          if (nc_close(ncid)) ERR;
 
          if (nc_open(file_names[format], NC_WRITE, &ncid)) ERR;
+         /* Should work but does not. Raname issues. */
          /* if (check_file(ncid, LAT, RH, TAL)) ERR; */
          if (nc_close(ncid)) ERR;
       }
       SUMMARIZE_ERR;
 
-      fprintf(stderr,"*** Test renaming just coordinate variable for %s...",
-              fmt_names[format]);
+      printf("*** Test renaming just coordinate variable for %s...",
+             fmt_names[format]);
       {
          if (create_test_file(file_names[format], formats[format])) ERR;
          if (nc_open(file_names[format], NC_WRITE, &ncid)) ERR;
          if (nc_inq_dimid(ncid, ODIM_NAME, &dimid)) ERR;
          if (nc_inq_varid(ncid, OVAR_NAME, &varid)) ERR;
          if (nc_inq_varid(ncid, OVAR2_NAME, &var2id)) ERR;
-         if (nc_redef(ncid)) ERR;  /* omitting this and nc_enddef call eliminates bug */
-         /* if (nc_rename_dim(ncid, dimid, NDIM_NAME)) ERR; */
+         if (nc_redef(ncid)) ERR;
          if (nc_rename_var(ncid, varid, NVAR_NAME)) ERR;
          if (nc_enddef(ncid)) ERR;
          if (nc_get_var_int(ncid, varid, lats_in)) ERR;
@@ -362,17 +463,16 @@ main(int argc, char **argv)
       }
       SUMMARIZE_ERR;
 
-      fprintf(stderr,"*** Test renaming just coordinate dimension for %s...",
-              fmt_names[format]);
+      printf("*** Test renaming just coordinate dimension for %s...",
+             fmt_names[format]);
       {
          if (create_test_file(file_names[format], formats[format])) ERR;
          if (nc_open(file_names[format], NC_WRITE, &ncid)) ERR;
          if (nc_inq_dimid(ncid, ODIM_NAME, &dimid)) ERR;
          if (nc_inq_varid(ncid, OVAR_NAME, &varid)) ERR;
          if (nc_inq_varid(ncid, OVAR2_NAME, &var2id)) ERR;
-         if (nc_redef(ncid)) ERR; /* omitting this and nc_enddef call eliminates bug */
+         if (nc_redef(ncid)) ERR;
          if (nc_rename_dim(ncid, dimid, NDIM_NAME)) ERR;
-         /* if (nc_rename_var(ncid, varid, NVAR_NAME)) ERR; */
          if (nc_enddef(ncid)) ERR;
          if (nc_get_var_int(ncid, varid, lats_in)) ERR;
          for (ii = 0; ii < DIM_LEN; ii++) {
