@@ -305,14 +305,14 @@ main(int argc, char **argv)
    if (!(comp_array_of_vlen_of_comp_out = calloc(sizeof(struct s3), DIM3_LEN))) ERR;
 
    /* Create some output data: a vlen of struct s1. */
-   for (i = 0; i < DIM1_LEN; i++)
+   for (size_t i = 0; i < DIM1_LEN; i++)
    {
       vlen_of_comp_out[i].len = i + 1;
       if (!(vlen_of_comp_out[i].p = malloc(sizeof(struct s1) * vlen_of_comp_out[i].len)))
 	 return NC_ENOMEM;
       for (j = 0; j < vlen_of_comp_out[i].len; j++)
       {
-	 ((struct s1 *)vlen_of_comp_out[i].p)[j].x = 42.42;
+	 ((struct s1 *)vlen_of_comp_out[i].p)[j].x = 42.42f;
 	 ((struct s1 *)vlen_of_comp_out[i].p)[j].y = 2.0;
       }
    }
@@ -322,13 +322,13 @@ main(int argc, char **argv)
    for (i = 0; i < DIM2_LEN; i++)
       for (j = 0; j < NUM_S1; j++)
       {
-	 comp_array_of_comp_out[i].data[j].x = 42.42;
+	 comp_array_of_comp_out[i].data[j].x = 42.42f;
 	 comp_array_of_comp_out[i].data[j].y = 2.0;
       }
 
    /* Create some output data: a struct which holds an array of
     * vlen of struct s1. */
-   for (i = 0; i < DIM3_LEN; i++)
+   for (size_t i = 0; i < DIM3_LEN; i++)
       for (j = 0; j < NUM_VL; j++)
       {
 	 comp_array_of_vlen_of_comp_out[i].data[j].len = i + 1;
@@ -336,7 +336,7 @@ main(int argc, char **argv)
 	    return NC_ENOMEM;
 	 for (k = 0; k < comp_array_of_vlen_of_comp_out[i].data[j].len; k++)
 	 {
-	    ((struct s1 *)comp_array_of_vlen_of_comp_out[i].data[j].p)[k].x = 42.42;
+	    ((struct s1 *)comp_array_of_vlen_of_comp_out[i].data[j].p)[k].x = 42.42f;
 	    ((struct s1 *)comp_array_of_vlen_of_comp_out[i].data[j].p)[k].y = 2.0;
 	 }
       }
