@@ -1,26 +1,26 @@
 /*********************************************************************
- *   Copyright 1993, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *********************************************************************/
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include "nc_tests.h"
 
 #define BUFLEN 100000
 
 int
 main(int argc, char** argv)
 {
-    long pos = 0;
     unsigned char buffer[BUFLEN];
     size_t count, red, avail, trunc;
     unsigned char* p = buffer;
-    long i;
+    size_t i;
     FILE* input = stdin;
 
     if(argc > 1) {
-	input = fopen(argv[1],"r");
+	input = NCfopen(argv[1],"r");
     }
 
     /* Read the whole file */
