@@ -26,14 +26,13 @@
 #include "ncuri.h"
 #include "ncbytes.h"
 #include "nclog.h"
+#include "ncpathmgr.h"
 
 #include "oc.h"
 #include "ocx.h"
 
-#ifdef _MSC_VER
+#if defined(_WIN32) && !defined(__MINGW32__)
 #include "XGetopt.h"
-int opterr, optind;
-char* optarg;
 #endif
 
 #ifndef nulldup
@@ -56,8 +55,6 @@ char* optarg;
 /*Mnemonic*/
 #define TOPLEVEL 1
 
-int ocdebug;
-
 static OCerror ocstat;
 static OClink glink;
 
@@ -130,7 +127,7 @@ static OCerror printdata_indices(OClink, OCdatanode, NCbytes*,int);
 static OCerror printdata_container(OClink, OCdatanode, NCbytes*,int);
 static OCerror printdata_leaf(OClink, OCdatanode, NCbytes*,int);
 
-static off_t odom_init(size_t rank, size_t* indices, size_t* dimsizes);
+static size_t odom_init(size_t rank, size_t* indices, size_t* dimsizes);
 static int odom_more(size_t rank, size_t* indices, size_t* dimsizes);
 static void odom_next(size_t rank, size_t* indices, size_t* dimsizes);
 
@@ -233,7 +230,7 @@ main(int argc, char **argv)
 	    } break;
         case 'X': {
 	    int c0;
-	    int so = (optarg == NULL ? 0 : strlen(optarg));
+	    size_t so = (optarg == NULL ? 0 : strlen(optarg));
 	    if(so == 0) usage("missing -X argument");
 	    c0 = optarg[0];
 	    switch (c0) {
@@ -246,7 +243,7 @@ main(int argc, char **argv)
             if(ocopt.output != NULL) fclose(ocopt.output);
 	    if(optarg == NULL)
 		usage("-o does not specify a file name");
-	    ocopt.output = fopen(optarg,"w");
+	    ocopt.output = NCfopen(optarg,"w");
             if(ocopt.output == NULL)
 		usage("-o file not writeable");
 	    break;
@@ -282,7 +279,7 @@ main(int argc, char **argv)
 
     if(ocopt.logging) {
 	ncloginit();
-	ncsetlogging(1);
+	ncsetloglevel(NCLOGNOTE);
 	if(!nclogopen(NULL))
 	    fprintf(stderr,"Failed to open logging output\n");
     }
@@ -559,8 +556,7 @@ printdata_container(OClink link, OCdatanode datanode, NCbytes* buffer, int istop
 	pushstack(field);
         FAIL(printdata_indices(link,field,buffer,istoplevel));
 	popstack();
-	if(oc_data_free(link,field) != OC_NOERR)
-	    break;
+	oc_data_free(link,field);
     }
     return stat;
 }
@@ -925,7 +921,7 @@ stringescape(char* s)
 {
     size_t len;
     char* p;
-    int c;
+    char c;
     char* escapedstring;
 
     if(s == NULL) return NULL;
@@ -952,7 +948,7 @@ static char*
 idescape(char* id, char* escapeid, size_t esize)
 {
     char* p;
-    int c;
+    char c;
 
     if(id == NULL) return NULL;
     p = escapeid;
@@ -1037,7 +1033,7 @@ dumpdatanode(OClink link, OCdatanode datanode, size_t count, void* memory, NCbyt
             ncbytescat(path,id);
             if(entry->rank > 0) {
 		for(i=0;i<entry->rank;i++) {
-                    sprintf(tmp,"[%lu]",(unsigned long)entry->indices[i]);
+                    snprintf(tmp,sizeof(tmp),"[%lu]",(unsigned long)entry->indices[i]);
                     ncbytescat(path,tmp);
 		}
             }
@@ -1051,7 +1047,7 @@ dumpdatanode(OClink link, OCdatanode datanode, size_t count, void* memory, NCbyt
         case OC_Sequence:
             ncbytescat(path,"/");
             ncbytescat(path,id);
-            sprintf(tmp,"[%lu]",(unsigned long)entry->indices[0]);
+            snprintf(tmp,sizeof(tmp),"[%lu]",(unsigned long)entry->indices[0]);
             ncbytescat(path,tmp);
             break;
 
@@ -1072,8 +1068,7 @@ dumpdatanode(OClink link, OCdatanode datanode, size_t count, void* memory, NCbyt
     ncbytescat(buffer,tmp);
     if(entry->rank > 0) {
 	if(ocopt.octest) { /* Match the octest output */
-	    off_t xproduct;
-	    xproduct = totaldimsize(entry->rank,entry->dimsizes);
+	    size_t xproduct = totaldimsize(entry->rank,entry->dimsizes);
             snprintf(tmp,sizeof(tmp),"[0..%lu]",(unsigned long)xproduct-1);
             ncbytescat(buffer,tmp);
 	} else {
@@ -1095,12 +1090,11 @@ dumpdatanode(OClink link, OCdatanode datanode, size_t count, void* memory, NCbyt
     return OC_NOERR;
 }
 
-static off_t
+static size_t
 odom_init(size_t rank, size_t* indices, size_t* dimsizes)
 {
-    int i;
-    off_t count;
-    for(count=1,i=0;i<rank;i++) {
+    size_t count = 1;
+    for(size_t i=0;i<rank;i++) {
         indices[i] = 0;
 	count *= dimsizes[i];
     }
@@ -1110,11 +1104,10 @@ odom_init(size_t rank, size_t* indices, size_t* dimsizes)
 static void
 odom_next(size_t rank, size_t* indices, size_t* dimsizes)
 {
-    int i;
-    for(i=rank-1;i>=0;i--) {
-	indices[i]++;
-	if(indices[i] < dimsizes[i]) break;
-	if(i > 0) indices[i] = 0;
+    for(size_t i = rank; i-->0;) {
+        indices[i]++;
+        if(indices[i] < dimsizes[i]) break;
+        if(i > 0) indices[i] = 0;
     }
 }
 
