@@ -1,6 +1,6 @@
 #!/bin/sh
 
-#set -e
+set -e
 
 if test "x$srcdir" = x ; then srcdir=`pwd`; fi
 . ../test_common.sh
