@@ -1,7 +1,7 @@
 #ifndef NC_GENLIB_H
 #define NC_GENLIB_H
 /*********************************************************************
- *   Copyright 1993, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *   $Header: /upc/share/CVS/netcdf-3/ncgen/genlib.h,v 1.20 2010/05/17 23:26:45 dmh Exp $
  *********************************************************************/
@@ -59,28 +59,31 @@ extern void nestedfqn(Symbol* sym);
 extern void attfqn(Symbol* sym);
 
 /* from: escapes.c */
-extern int unescape(char*, const char*, int, int);
+extern int unescape(const char*, size_t, int, char**);
 extern int unescapeoct(const char* s);
 extern int unescapehex(const char* s);
-extern char* cescapifychar(unsigned int c, int quote);
+extern char* cescapifychar(char c, int quote);
 extern char* codify(const char *name);
-extern char* escapifychar(unsigned int c, char* s0, int quote);
-extern char* escapify(char*,int,size_t);
-extern char* escapifyname(char* s0);
+extern char* escapifychar(char c, char* s0, int quote);
+extern char* escapify(const char*,int,size_t);
+extern char* escapifyname(const char* s0);
 extern void cquotestring(Bytebuffer*,char quote);
 extern void f77quotestring(Bytebuffer*);
 extern char* xescapify(char* s0, int quote, size_t len);
 extern char* jescapify(char* s0, int quote, size_t len);
 extern char* jescapifyname(char* s0);
 extern char* fqnescape(const char* s);
+extern char* esc_strchr(char* s, int c, int octhex);
 
 /* from: getfill.c */
-extern void nc_getfill(NCConstant*);
+extern void nc_getfill(NCConstant*,Symbol*);
 extern char* nc_dfaltfillname(nc_type);
 extern struct Datalist* getfiller(Symbol*); /* symbol isa variable|type */
 
 /* from: ncgen.y */
 extern Symbol* install(const char *sname);
+extern Symbol* installin(const char *sname,Symbol* grp);
+extern void freesymbol(Symbol*);
 extern Symbol* basetypefor(nc_type nctype);/* Convert nctype to a Symbol*/
 extern Symbol* makearraytype(Symbol*, Dimset*);
 
@@ -112,15 +115,18 @@ extern char* indented(int n);
 #ifdef ENABLE_BINARY
 /* from: genbin.c */
 extern Generator* bin_generator;
-extern void gen_netcdf(const char *filename);
-extern void cl_netcdf(void);
+extern void genbin_netcdf(void);
+extern void genbin_close(void);
+/* from: bindata.c */
+extern int binary_generate_data(Datalist* data, Symbol* tsym, Datalist* fillvalue, Bytebuffer* databuf);
+extern int binary_reclaim_data(Symbol* tsym, void* memory, size_t count);
 #endif
 
 #ifdef ENABLE_C
 /* from: genc.c */
 extern Generator* c_generator;
-extern void gen_ncc(const char *filename);
-extern void cl_c(void);
+extern void genc_netcdf(void);
+extern void genc_close(void);
 extern const char* ctypename(Symbol*);
 extern const char* nctype(nc_type type);
 extern const char* ncctype(nc_type type);
@@ -132,8 +138,8 @@ extern const char* cname(Symbol* sym);
 #ifdef ENABLE_F77
 /* from: genf77.c */
 extern Generator* f77_generator;
-extern void gen_ncf77(const char *filename);
-extern void cl_f77(void);
+extern void genf77_netcdf(void);
+extern void genf77_close(void);
 extern const char* f77name(Symbol*);
 extern const char* f77typename(Symbol*);
 #endif
@@ -141,8 +147,8 @@ extern const char* f77typename(Symbol*);
 #ifdef ENABLE_JAVA
 /* from: genj.c */
 extern Generator* j_generator;
-extern void gen_ncjava(const char *filename);
-extern void cl_java(void);
+extern void genjava_netcdf(void);
+extern void genjava_close(void);
 extern void jpartial(char*);
 extern void jline(char*);
 extern void jlined(int,char*);
@@ -159,11 +165,12 @@ extern int specials_flag; /* 1 => special attributes are present */
 extern int usingclassic;   /* 1 => k_flag == 1|2|5 */
 extern int k_flag;
 extern int ncloglevel;
+extern int wholevarsize;
 extern GlobalSpecialData globalspecials;
 
 /* Global data */
 
-extern Symbol* symlist;      /* all symbol objects created */
+extern List* symlist;      /* all symbol objects created */
 extern Symbol* rootgroup;
 
 /* Track definitions of dims, types, attributes, and vars*/
@@ -187,18 +194,20 @@ extern char* binary_ext;
 extern int nofill_flag;
 extern int header_only;
 extern char* mainname;
-extern size_t nciterbuffersize;
 
 extern char* progname; /* for error messages*/
 extern char *netcdf_name; /* command line -o file name */
 extern char *datasetname; /* name from the netcdf <name> {} */
 extern char *cdlname; /* name from the command line */
 
-/* from: util.c */
-extern void* emalloc (size_t);
-extern void* ecalloc (size_t);
-extern void* erealloc(void*,size_t);
+extern size_t zerosvector[NC_MAX_VAR_DIMS];
+extern size_t onesvector[NC_MAX_VAR_DIMS];
 
 extern const char* specialname(int tag);
 
+extern void init_netcdf(void);
+extern void finalize_netcdf(int);
+extern void parse_init(void);
+extern int ncgparse(void);
+
 #endif /*!NC_GENLIB_H*/
