@@ -1,5 +1,5 @@
 /*********************************************************************
- *   Copyright 2009, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *********************************************************************/
 /* $Id: odom.c,v 1.5 2010/05/27 21:34:18 dmh Exp $ */
@@ -70,7 +70,7 @@ odometerprint(Odometer* odom)
 	strcat(line,"[]");
     } else for(i=0;i<odom->rank;i++) {
 	int ioffset = i + odom->offset;
-	sprintf(tmp,"[%lu/%lu..%lu:%lu]",
+	snprintf(tmp,sizeof(tmp),"[%lu/%lu..%lu:%lu]",
 		(unsigned long)odom->origin->index[ioffset],
 		(unsigned long)odom->origin->start[ioffset],
 		(unsigned long)odom->origin->declsize[ioffset],
