@@ -1,19 +1,17 @@
-/* Copyright 2009, UCAR/Unidata and OPeNDAP, Inc.
+/* Copyright 2018, UCAR/Unidata and OPeNDAP, Inc.
    See the COPYRIGHT file for more information. */
 
 #include "config.h"
 #include "ocinternal.h"
 #include "occompile.h"
 #include "ocdebug.h"
-
-static const unsigned int MAX_UINT = 0xffffffff;
+#include <stddef.h>
 
 static OCerror mergedas1(OCnode* dds, OCnode* das);
 static OCerror mergedods1(OCnode* dds, OCnode* das);
 static OCerror mergeother1(OCnode* root, OCnode* das);
 static char* pathtostring(NClist* path, char* separator);
 static void computefullname(OCnode* node);
-static OCerror mergeother1(OCnode* root, OCnode* das);
 static OCerror mergeother(OCnode* ddsroot, NClist* dasnodes);
 
 /* Process ocnodes to fix various semantic issues*/
@@ -81,23 +79,23 @@ computefullname(OCnode* node)
 static char*
 pathtostring(NClist* path, char* separator)
 {
-    int slen,i,len;
+    size_t slen,i,len;
     char* pathname;
     if(path == NULL) return NULL;
     len = nclistlength(path);
     if(len == 0) return NULL;
     for(i=0,slen=0;i<len;i++) {
-	OCnode* node = (OCnode*)nclistget(path,(size_t)i);
+	OCnode* node = (OCnode*)nclistget(path, i);
 	if(node->container == NULL || node->name == NULL) continue;
 	slen += strlen(node->name);
     }
     slen += ((len-1)*strlen(separator));
     slen += 1;   /* for null terminator*/
-    pathname = (char*)ocmalloc((size_t)slen);
+    pathname = (char*)ocmalloc(slen);
     MEMCHECK(pathname,NULL);
     pathname[0] = '\0';
     for(i=0;i<len;i++) {
-	OCnode* node = (OCnode*)nclistget(path,(size_t)i);
+	OCnode* node = (OCnode*)nclistget(path, i);
 	if(node->container == NULL || node->name == NULL) continue;
 	if(strlen(pathname) > 0) strcat(pathname,separator);
         strcat(pathname,node->name);
@@ -365,14 +363,15 @@ mergedas1(OCnode* dds, OCnode* das)
     for(i=0;i<nclistlength(das->subnodes);i++) {
 	OCnode* attnode = (OCnode*)nclistget(das->subnodes,i);
 	if(attnode->octype == OC_Attribute) {
+            OCattribute* att;
 	    if(dds->octype == OC_Atomic
 		|| dds->octype == OC_Sequence
 		|| dds->octype == OC_Structure
 		|| dds->octype == OC_Grid)
 	        attnode->att.var = dds;
-	    OCattribute* att = makeattribute(attnode->name,
-						attnode->etype,
-						attnode->att.values);
+            att = makeattribute(attnode->name,
+                                attnode->etype,
+                                attnode->att.values);
             nclistpush(dds->attributes,(void*)att);
 	}
     }
@@ -399,13 +398,13 @@ mergedods1(OCnode* dds, OCnode* dods)
             */
 	    size_t len =   strlen(attnode->name)
                          + strlen(dods->name)
-			 + strlen(".")
-			 + 1; /*null*/
-	    char* newname = (char*)malloc(len);
+			 + strlen(".");
+	    len++; /*strlcat nul*/
+	    char* newname = (char*)malloc(len+1);
 	    if(newname == NULL) return OC_ENOMEM;
-	    strcpy(newname,dods->name);
-	    strcat(newname,".");
-	    strcat(newname,attnode->name);
+	    strncpy(newname,dods->name,len);
+	    strlcat(newname,".",len);
+	    strlcat(newname,attnode->name,len);
 	    att = makeattribute(newname,attnode->etype,attnode->att.values);
 	    free(newname);
             nclistpush(dds->attributes,(void*)att);
@@ -418,7 +417,7 @@ static OCerror
 mergeother(OCnode* ddsroot, NClist* dasnodes)
 {
     OCerror stat = OC_NOERR;
-    int i;
+    size_t i;
     for(i=0;i<nclistlength(dasnodes);i++) {
 	OCnode* das = (OCnode*)nclistget(dasnodes,i);
 	if(das == NULL) continue;
@@ -446,7 +445,7 @@ mergeother1(OCnode* root, OCnode* das)
         att = makeattribute(das->fullname,das->etype,das->att.values);	
         nclistpush(root->attributes,(void*)att);
     } else if(das->octype == OC_Attributeset) {
-	int i;
+	size_t i;
 	/* Recurse */
         for(i=0;i<nclistlength(das->subnodes);i++) {
 	    OCnode* sub = (OCnode*)nclistget(das->subnodes,i);
@@ -483,7 +482,7 @@ occorrelater(OCnode* dds, OCnode* dxd)
     if(dxd->name != NULL && dxd->name != NULL
        && strcmp(dxd->name,dds->name) != 0) {
 	OCTHROWCHK((ocstat = OC_EINVAL)); goto fail;
-    } else if(dxd->name != dxd->name) { /* test NULL==NULL */
+    } else if(dxd->name != dds->name) { /* test NULL==NULL */
 	OCTHROWCHK((ocstat = OC_EINVAL)); goto fail;
     }
 
